@@ -1,56 +1,133 @@
 import json
 import os
-
-from flask import Blueprint, Response, request, stream_with_context, current_app
+from urllib.parse import urlparse
 
 import cirrocumulus.data_processing as data_processing
-from .dataset_api import DatasetAPI
-from .envir import CIRRO_SERVE, CIRRO_FOOTER, CIRRO_UPLOAD, CIRRO_BRAND, CIRRO_EMAIL, CIRRO_AUTH, CIRRO_DATABASE, \
-    CIRRO_DATASET_SELECTOR_COLUMNS, CIRRO_CELL_ONTOLOGY, CIRRO_STATIC_DIR, CIRRO_MOUNT, CIRRO_MIXPANEL
-from .invalid_usage import InvalidUsage
-from .job_api import submit_job
-from .util import json_response, get_scheme, get_fs
+from flask import Blueprint, Response, request, stream_with_context
 
-blueprint = Blueprint('blueprint', __name__)
+from .anndata_util import adata_to_df
+from .blueprint_util import get_database, map_url, get_auth
+from .dataset_api import DatasetAPI
+from .envir import CIRRO_SERVE, CIRRO_FOOTER, CIRRO_UPLOAD, CIRRO_BRAND, CIRRO_EMAIL, CIRRO_DATASET_SELECTOR_COLUMNS, \
+    CIRRO_CELL_ONTOLOGY, CIRRO_STATIC_DIR, CIRRO_MIXPANEL, CIRRO_AUTH_CLIENT_ID, CIRRO_AUTH_BASE_URL, CIRRO_LIBRARY, \
+    CIRRO_SPECIES
+from .invalid_usage import InvalidUsage
+from .job_api import submit_job, delete_job
+from .util import json_response, get_scheme, get_fs, open_file
+
+cirro_blueprint = Blueprint('cirro', __name__)
 
 dataset_api = DatasetAPI()
 
-remapped_urls = dict()
-if os.environ.get(CIRRO_MOUNT) is not None:
-    tokens = os.environ.get(CIRRO_MOUNT).split(',')
-    for token in tokens:
-        bucket, local_path = token.split(':')
-        remapped_urls[bucket] = local_path
+
+def get_email_and_dataset(content):
+    email = get_auth().auth()['email']
+    dataset_id = content.get('id', '')
+    if dataset_id == '':
+        return 'Please supply an id', 400
+    database_api = get_database()
+    dataset = database_api.get_dataset(email, dataset_id)
+    dataset['url'] = map_url(dataset['url'])
+    return email, dataset
 
 
-@blueprint.errorhandler(InvalidUsage)
+def upload_file(file):
+    from werkzeug.utils import secure_filename
+    filename = secure_filename(file.filename)
+    dest = os.path.join(os.environ.get(CIRRO_UPLOAD), filename)
+    dest_fs = get_fs(dest).open(dest, mode='wb')
+    file.save(dest_fs)
+    dest_fs.close()
+    return dest
+
+
+def copy_url(url):
+    from werkzeug.utils import secure_filename
+    upload = os.environ.get(CIRRO_UPLOAD)
+
+    if url.endswith('/') or os.path.isdir(url):  # don't copy directories
+        return url
+
+    if urlparse(upload).netloc == urlparse(url).netloc:  # don't copy if already in the same bucket
+        return url
+    src_scheme = get_scheme(url)
+    src_fs = get_fs(src_scheme)
+    filename = secure_filename(os.path.basename(url))
+    dest = os.path.join(upload, filename)
+    dest_scheme = get_scheme(dest)
+    dest_fs = get_fs(dest_scheme)
+    out = dest_fs.open(dest, 'wb')
+    n = 1024 * 1024
+    with src_fs.open(url, mode='rb') as r:
+        while True:
+            b = r.read(n)
+            if not b:
+                break
+            out.write(b)
+    out.close()
+    return dest
+
+
+def get_file_path(file, dataset_url):
+    # when serving dataset, file must be relative to dataset directory
+    if os.environ.get(CIRRO_SERVE) == 'true':
+        _, ext = os.path.splitext(dataset_url)
+        if file[0] == '/' or file.find('..') != -1:
+            raise ValueError('Incorrect path')
+        file_path = os.path.join(dataset_url, file)
+    else:
+        file_path = file
+        scheme = get_scheme(file_path)
+        if scheme == 'file' and not os.path.exists(file_path):
+            file_path = os.path.join(dataset_url, file)
+    return file_path
+
+
+def send_file(file_path):
+    import mimetypes
+    mimetype, encoding = mimetypes.guess_type(file_path)
+    chunk_size = 4096
+    f = get_fs(file_path).open(file_path)
+
+    def generate():
+        while True:
+            chunk = f.read(chunk_size)
+            if len(chunk) <= 0:
+                f.close()
+                break
+            yield chunk
+
+    r = Response(stream_with_context(generate()), mimetype=mimetype)
+    r.headers["Content-Encoding"] = encoding
+    return r
+
+
+@cirro_blueprint.errorhandler(InvalidUsage)
 def handle_invalid_usage(error):
     return Response(error.message, error.status_code)
 
 
-def get_database():
-    return current_app.config[CIRRO_DATABASE]
+def load_json(key):
+    if os.environ.get(key) is not None:
+        if get_fs(os.environ[key]).exists(os.environ[key]):
+            with open_file(os.environ[key], 'rt') as f:
+                return json.load(f)
+        else:
+            return json.loads(os.environ.get(key))
 
 
-def get_auth():
-    return current_app.config[CIRRO_AUTH]
-
-
-@blueprint.route('/server', methods=['GET'])
+@cirro_blueprint.route('/server', methods=['GET'])
 def handle_server():
     # no login required
     d = {}
     d['email'] = os.environ.get(CIRRO_EMAIL)
     d['capabilities'] = get_database().capabilities()
-    d['clientId'] = get_auth().client_id
+    d['clientId'] = os.environ.get(CIRRO_AUTH_CLIENT_ID)
+    d['baseUrl'] = os.environ.get(CIRRO_AUTH_BASE_URL)
     if os.environ.get(CIRRO_MIXPANEL) is not None:
         d['mixpanel'] = os.environ[CIRRO_MIXPANEL]
-    if os.environ.get(CIRRO_DATASET_SELECTOR_COLUMNS) is not None:
-        if os.path.exists(os.environ[CIRRO_DATASET_SELECTOR_COLUMNS]):
-            with open(os.environ[CIRRO_DATASET_SELECTOR_COLUMNS], 'rt') as f:
-                d['datasetSelectorColumns'] = json.loads(f.read())
-        else:
-            d['datasetSelectorColumns'] = json.loads(os.environ[CIRRO_DATASET_SELECTOR_COLUMNS])
+    d['datasetSelectorColumns'] = load_json(CIRRO_DATASET_SELECTOR_COLUMNS)
+
     if os.environ.get(CIRRO_FOOTER) is not None:
         with open(os.environ.get(CIRRO_FOOTER), 'rt') as f:
             d['footer'] = f.read()
@@ -58,8 +135,33 @@ def handle_server():
         with open(os.environ.get(CIRRO_BRAND), 'rt') as f:
             d['brand'] = f.read()
     d['upload'] = os.environ.get(CIRRO_UPLOAD) is not None
+    d['species'] = load_json(CIRRO_SPECIES) or dict(favorite=["Homo sapiens", "Mus musculus"],
+                                                    other=["Gallus gallus", "Macaca fascicularis",
+                                                           "Macaca mulatta", "Rattus norvegicus"])
+
+    # from https://www.ebi.ac.uk/ols/ontologies/efo/terms?iri=http%3A%2F%2Fwww.ebi.ac.uk%2Fefo%2FEFO_0010183&viewMode=All&siblings=false
+    d['library'] = load_json(CIRRO_LIBRARY) or ["10x 3' v1", "10x 3' v2", "10x 3' v3",
+                                                "10x 5' v1", "10x 5' v2",
+                                                "10x feature barcode (CRISPR screening)",
+                                                "10x feature barcode (cell surface protein profiling)",
+                                                "10x feature barcode (sample multiplexing)",
+                                                "10x Ig enrichment", "10x TCR enrichment",
+                                                "10x multiome", "10x scATAC-seq",
+                                                "Visium Spatial Gene Expression", 'CEL-seq', 'CEL-seq2',
+                                                'CITE-seq (cell surface protein profiling)',
+                                                'CITE-seq (sample multiplexing)', 'DroNc-seq', 'Drop-seq',
+                                                'Fluidigm C1-based library preparation', 'MARS-seq', 'Nx1-seq',
+                                                'Quartz-seq', 'SCOPE-chip', 'mcSCRB-seq', 'SPLiT-seq', 'STRT-seq',
+                                                'Seq-Well S3', 'Smart-seq',
+                                                'Smart-seq2', 'barcoded plate-based single cell RNA-seq', 'inDrop',
+                                                'mCT-seq', 'microwell-seq', '10x multiome', '10x scATAC-seq',
+                                                'scATAC-seq (Microfluidics)',
+                                                'scATAC-seq (cell index)', 'scBS-seq', 'scChIP-seq', 'sci-CAR',
+                                                'sci-Plex', 'sci-RNA-seq3',
+                                                'single cell Hi-C', 'snmC-Seq2']
+
     if os.environ.get(CIRRO_CELL_ONTOLOGY) is not None:
-        if os.path.exists(os.environ[CIRRO_CELL_ONTOLOGY]):
+        if get_fs(os.environ[CIRRO_CELL_ONTOLOGY]).exists(os.environ[CIRRO_CELL_ONTOLOGY]):
             def parse_term(f):
                 term = dict()
                 for term_line in f:
@@ -75,7 +177,7 @@ def handle_server():
                 return term
 
             terms = []
-            with open(os.environ[CIRRO_CELL_ONTOLOGY], 'rt') as f:  # obo file
+            with open_file(os.environ[CIRRO_CELL_ONTOLOGY], 'rt') as f:  # obo file
                 for line in f:
                     line = line.strip()
                     if line == "[Term]":
@@ -89,7 +191,7 @@ def handle_server():
     return json_response(d)
 
 
-@blueprint.route('/category_name', methods=['GET', 'PUT'])
+@cirro_blueprint.route('/category_name', methods=['GET', 'PUT'])
 def handle_category_name():
     """CRUD for category name.
     """
@@ -125,7 +227,7 @@ def handle_category_name():
         return '', 200
 
 
-@blueprint.route('/feature_set', methods=['POST', 'DELETE'])
+@cirro_blueprint.route('/feature_set', methods=['POST', 'DELETE'])
 def handle_feature_set():
     """CRUD for a feature set.
     """
@@ -164,7 +266,7 @@ def handle_feature_set():
         return json_response('', 204)
 
 
-@blueprint.route('/views', methods=['GET'])
+@cirro_blueprint.route('/views', methods=['GET'])
 def handle_dataset_views():
     """List available views for a dataset.
     """
@@ -177,7 +279,7 @@ def handle_dataset_views():
     return json_response(get_database().dataset_views(email, dataset_id))
 
 
-@blueprint.route('/view', methods=['GET', 'POST', 'PUT', 'DELETE'])
+@cirro_blueprint.route('/view', methods=['GET', 'POST', 'PUT', 'DELETE'])
 def handle_dataset_view():
     """CRUD for a dataset view.
     """
@@ -199,17 +301,17 @@ def handle_dataset_view():
         if request.method == 'POST' and dataset_id is None:
             return 'Please supply a ds_id', 400
 
-        view_id = database_api.upsert_dataset_view(
+        result = database_api.upsert_dataset_view(
             email=email,
             dataset_id=dataset_id,
             view=d)
-        return json_response({'id': view_id})
+        return json_response(result)
     elif request.method == 'DELETE':
         database_api.delete_dataset_view(email, view_id=d.pop('id'))
         return json_response('', 204)
 
 
-@blueprint.route('/schema', methods=['GET'])
+@cirro_blueprint.route('/schema', methods=['GET'])
 def handle_schema():
     """Get dataset schema.
     """
@@ -217,50 +319,14 @@ def handle_schema():
     database_api = get_database()
     dataset_id = request.args.get('id', '')
     dataset = database_api.get_dataset(email, dataset_id)
-    schema = dataset_api.get_schema(dataset)
-    schema.update(dataset)  # add title, etc from database to schema
+    dataset['url'] = map_url(dataset['url'])
+    schema = dataset  # dataset has title, etc. from database
     schema['markers'] = database_api.get_feature_sets(email=email, dataset_id=dataset_id)
+    schema.update(dataset_api.get_schema(dataset))
     return json_response(schema)
 
 
-def get_file_path(file, dataset_url):
-    # when serving dataset, file must be relative to dataset directory
-    if os.environ.get(CIRRO_SERVE) == 'true':
-        _, ext = os.path.splitext(dataset_url)
-        if file[0] == '/' or file.find('..') != -1:
-            raise ValueError('Incorrect path')
-        file_path = os.path.join(dataset_url, file)
-    else:
-        file_path = file
-        scheme = get_scheme(file_path)
-        if scheme == 'file' and not os.path.exists(file_path):
-            file_path = os.path.join(dataset_url, file)
-    return file_path
-
-
-def send_file(file_path):
-    import mimetypes
-    mimetype, encoding = mimetypes.guess_type(file_path)
-    # with dataset_api.fs_adapter.get_fs(file_path).open(file_path) as f:
-    #     bytes = f.read()
-    # return Response(bytes, mimetype=mimetype[0])
-    chunk_size = 4096
-    f = get_fs(file_path).open(file_path)
-
-    def generate():
-        while True:
-            chunk = f.read(chunk_size)
-            if len(chunk) <= 0:
-                f.close()
-                break
-            yield chunk
-
-    r = Response(stream_with_context(generate()), mimetype=mimetype)
-    r.headers["Content-Encoding"] = encoding
-    return r
-
-
-@blueprint.route('/file', methods=['GET'])
+@cirro_blueprint.route('/file', methods=['GET'])
 def handle_file():
     email = get_auth().auth()['email']
     database_api = get_database()
@@ -281,7 +347,7 @@ def handle_file():
     return send_file(file_path)
 
 
-@blueprint.route('/user', methods=['GET'])
+@cirro_blueprint.route('/user', methods=['GET'])
 def handle_user():
     email = get_auth().auth()['email']
     database_api = get_database()
@@ -289,27 +355,7 @@ def handle_user():
     return json_response(user)
 
 
-def map_url(url):
-    for remote_path in remapped_urls:
-        before_replace = url
-        url = url.replace(remote_path, remapped_urls[remote_path])
-        if before_replace != url:  # only replace URL once
-            return url
-    return url
-
-
-def get_email_and_dataset(content):
-    email = get_auth().auth()['email']
-    dataset_id = content.get('id', '')
-    if dataset_id == '':
-        return 'Please supply an id', 400
-    database_api = get_database()
-    dataset = database_api.get_dataset(email, dataset_id)
-    dataset['url'] = map_url(dataset['url'])
-    return email, dataset
-
-
-@blueprint.route('/data', methods=['POST'])
+@cirro_blueprint.route('/data', methods=['POST'])
 def handle_data():
     json_request = request.get_json(cache=False)
     email, dataset = get_email_and_dataset(json_request)
@@ -323,7 +369,7 @@ def handle_data():
                                     selection=json_request.get('selection')))
 
 
-@blueprint.route('/selection', methods=['POST'])
+@cirro_blueprint.route('/selection', methods=['POST'])
 def handle_selection():
     # selection includes stats, coordinates
     content = request.get_json(cache=False)
@@ -336,7 +382,7 @@ def handle_selection():
                                          embeddings=content.get('embeddings', [])))
 
 
-@blueprint.route('/selected_ids', methods=['POST'])
+@cirro_blueprint.route('/selected_ids', methods=['POST'])
 def handle_selected_ids():
     content = request.get_json(cache=False)
     email, dataset = get_email_and_dataset(content)
@@ -346,51 +392,14 @@ def handle_selected_ids():
 
 
 # List available datasets
-@blueprint.route('/datasets', methods=['GET'])
+@cirro_blueprint.route('/datasets', methods=['GET'])
 def handle_datasets():
     email = get_auth().auth()['email']
     database_api = get_database()
     return json_response(database_api.datasets(email))
 
 
-def upload_file(file):
-    from werkzeug.utils import secure_filename
-    filename = secure_filename(file.filename)
-    dest = os.path.join(os.environ.get(CIRRO_UPLOAD), filename)
-    dest_fs = get_fs(dest).open(dest, mode='wb')
-    file.save(dest_fs)
-    dest_fs.close()
-    return dest
-
-
-def copy_url(url):
-    from werkzeug.utils import secure_filename
-    upload = os.environ.get(CIRRO_UPLOAD)
-
-    if url.endswith('/') or os.path.isdir(url):  # don't copy directories
-        return url
-    from urllib.parse import urlparse
-    if urlparse(upload).netloc == urlparse(url).netloc:  # don't copy if already in the same bucket
-        return url
-    src_scheme = get_scheme(url)
-    src_fs = get_fs(src_scheme)
-    filename = secure_filename(os.path.basename(url))
-    dest = os.path.join(upload, filename)
-    dest_scheme = get_scheme(dest)
-    dest_fs = get_fs(dest_scheme)
-    out = dest_fs.open(dest, 'wb')
-    n = 1024 * 1024
-    with src_fs.open(url, mode='rb') as r:
-        while True:
-            b = r.read(n)
-            if not b:
-                break
-            out.write(b)
-    out.close()
-    return dest
-
-
-@blueprint.route('/dataset', methods=['GET', 'POST', 'PUT', 'DELETE'])
+@cirro_blueprint.route('/dataset', methods=['GET', 'POST', 'PUT', 'DELETE'])
 def handle_dataset():
     email = get_auth().auth()['email']
     database_api = get_database()
@@ -401,8 +410,11 @@ def handle_dataset():
         else:
             d = dict()
             for key in request.form:
-                d[key] = request.form[key]
-
+                val = request.form[key]
+                if len(val) >= 2 and val[0] == '[' and val[len(val) - 1] == ']':
+                    d[key] = json.loads(val)
+                else:
+                    d[key] = val
         dataset_id = d.get('id')
         dataset_name = d.get('name')
         url = d.get('url')  # e.g. gs://foo/a/b/
@@ -410,9 +422,6 @@ def handle_dataset():
         file = None
         if request.content_type != 'application/json':
             file = request.files.get('file')  # file upload
-        if readers is not None and not isinstance(readers, list):
-            import json
-            readers = json.loads(readers)
         if request.method == 'PUT' and dataset_id is None:  # update
             return 'Please supply an id', 400
         if request.method == 'POST' and dataset_name == '':  # new
@@ -441,24 +450,7 @@ def handle_dataset():
         return json_response(database_api.get_dataset(email, dataset_id, True))
 
 
-@blueprint.route('/job_status', methods=['GET'])
-def handle_job_status():
-    email = get_auth().auth()['email']
-    database_api = get_database()
-    job_id = request.args.get('id', '')
-    return database_api.get_job(email=email, job_id=job_id, return_result=False)
-
-
-@blueprint.route('/module', methods=['POST'])
-def handle_module_score():
-    content = request.get_json(cache=False)
-    email, dataset = get_email_and_dataset(content)
-    features = content.get('features')
-    adata = dataset_api.read_dataset(dataset, keys=dict(X=features))
-    return json_response(adata.X.mean(axis=1))
-
-
-@blueprint.route('/job', methods=['GET', 'DELETE'])
+@cirro_blueprint.route('/job', methods=['GET', 'DELETE', 'POST'])
 def handle_job():
     email = get_auth().auth()['email']
     database_api = get_database()
@@ -466,61 +458,73 @@ def handle_job():
         content = request.get_json(force=True, cache=False)
         job_id = content.get('id', '')
         database_api.delete_job(email, job_id)
+        delete_job(job_id)
         return json_response('', 204)
+    elif request.method == 'POST':
+        if os.environ.get('GAE_APPLICATION') is None:  # TODO
+            content = request.get_json(force=True, cache=False)
+            email, dataset = get_email_and_dataset(content)
+            params = content.get('params')
+            job_type = content.get('type')
+            job_name = content.get('name')
+            return dict(id=submit_job(database_api=database_api, dataset_api=dataset_api, email=email, dataset=dataset,
+                                      job_name=job_name, job_type=job_type, params=params))
+        else:
+            raise ValueError('Submit job not supported on GAE')
     else:
-        job_id = request.args.get('id', '')
-        if job_id.startswith('cirro-'):  # precomputed result
-            dataset_id = request.args.get('ds_id', '')
+        job_id = request.args['id']
+        c = request.args['c']
+        is_precomputed = job_id.startswith('cirro-')
+        if c == 'status' or c == 'params':
+            if is_precomputed:
+                job = dict(status='complete') if c == 'status' else dict()
+            else:
+                job = database_api.get_job(email=email, job_id=job_id, return_type=c)
+            if job is None:
+                return json_response('', 404)  # job deleted
+            return json_response(job, 200)
+        if c != 'result':
+            raise ValueError('c must be one of status, params, or result')
+        if is_precomputed:  # precomputed result
             email = get_auth().auth()['email']
-            dataset = database_api.get_dataset(email, dataset_id)
+            suggested_dataset_id = request.args['ds']
+            dataset = database_api.get_dataset(email, suggested_dataset_id)
+            # precomputed results need to be a child of dataset
             dataset['url'] = map_url(dataset['url'])
-            result = dataset_api.get_result(dataset, job_id)
-            return send_file(result)
-        job = database_api.get_job(email=email, job_id=job_id, return_result=True)
+            job_result = dataset_api.get_result(dataset, job_id)
+            if get_scheme(job_result) == 'file' and not os.path.exists(job_result):
+                return Response(job_result, content_type='application/json')
+            else:
+                return send_file(job_result)
+        job = database_api.get_job(email=email, job_id=job_id, return_type=c)
+        if job is None:
+            return json_response('', 404)  # job deleted
+        import anndata
         if isinstance(job, dict) and 'url' in job:
             url = job['url']
             content_type = job.get('content-type')
             if content_type == 'application/h5ad' or content_type == 'application/zarr':
-                import anndata
                 if content_type == 'application/h5ad':
                     with get_fs(url).open(url, mode='rb') as f:
                         adata = anndata.read(f)
                 else:
                     adata = anndata.read_zarr(get_fs(url).get_mapper(url))
-
-                import pandas as pd
-                # output = StringIO()
-                # adata2gct(adata, output)
-                # r = Response(output.getvalue(), mimetype='text/plain')
-                df = pd.DataFrame(adata.X, index=adata.obs.index, columns=adata.var.index)
-                df = df.T.join(adata.var)
-                df.index.name = 'id'
-                return Response(df.reset_index().to_json(double_precision=2, orient='records'),
-                                content_type='application/json')
+                adata_df = adata_to_df(adata)
+                return Response(adata_df.to_json(double_precision=2, orient='records'), content_type='application/json')
             else:
                 # URL to JSON or text
                 return send_file(url)
-        # elif isinstance(job, bytes):
-        #     job = job.decode('ascii')
+        elif isinstance(job, dict):
+            return json_response(job)
+        elif isinstance(job, anndata.AnnData):
+            return Response(adata_to_df(job).to_json(double_precision=2, orient='records'),
+                            content_type='application/json')
         return job
 
 
-@blueprint.route('/jobs', methods=['GET'])
+@cirro_blueprint.route('/jobs', methods=['GET'])
 def handle_jobs():
     email = get_auth().auth()['email']
     database_api = get_database()
     ds_id = request.args.get('id', '')
     return json_response(database_api.get_jobs(email=email, dataset_id=ds_id))
-
-
-@blueprint.route('/submit_job', methods=['POST'])
-def handle_submit_job():
-    if os.environ.get('GAE_APPLICATION') is None:  # TODO
-        database_api = get_database()
-        content = request.get_json(force=True, cache=False)
-        email, dataset = get_email_and_dataset(content)
-        params = content.get('params')
-        job_type = content.get('type')
-        job_name = content.get('name')
-        return dict(id=submit_job(database_api=database_api, dataset_api=dataset_api, email=email, dataset=dataset,
-                                  job_name=job_name, job_type=job_type, params=params))
