@@ -3,13 +3,16 @@ import os
 import anndata
 
 from cirrocumulus.anndata_dataset import AnndataDataset
-from cirrocumulus.envir import CIRRO_DATABASE, CIRRO_AUTH, CIRRO_JOB_TYPE, CIRRO_JOB_RESULTS, CIRRO_CELL_ONTOLOGY
-from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, SPATIAL_HELP
+from cirrocumulus.envir import CIRRO_DATABASE, CIRRO_AUTH, CIRRO_JOB_TYPE, \
+    CIRRO_JOB_RESULTS, CIRRO_CELL_ONTOLOGY
+from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, \
+    SPATIAL_HELP
 from cirrocumulus.local_db_api import LocalDbAPI
 from cirrocumulus.util import get_fs
 
 
-def configure_app(app, list_of_dataset_paths, spatial_directories, marker_paths):
+def configure_app(app, list_of_dataset_paths, spatial_directories,
+                  marker_paths):
     from cirrocumulus.api import dataset_api
     from cirrocumulus.no_auth import NoAuth
     try:
@@ -33,11 +36,15 @@ def configure_app(app, list_of_dataset_paths, spatial_directories, marker_paths)
         if len(dataset_paths) > 1:
             datasets = []
             for i in range(len(dataset_paths)):
-                dataset = anndata_dataset.get_data(get_fs(dataset_paths[i]), dataset_paths[i])
+                dataset = anndata_dataset.get_data(get_fs(dataset_paths[i]),
+                                                   dataset_paths[i])
                 if 'group' not in dataset.var:
-                    dataset.var['group'] = dataset.uns.get('name', 'dataset {}'.format(i + 1))
+                    dataset.var['group'] = dataset.uns.get('name',
+                                                           'dataset {}'.format(
+                                                               i + 1))
                 datasets.append(dataset)
-            adata = anndata.concat(datasets, axis=1, label='group', merge='unique')
+            adata = anndata.concat(datasets, axis=1, label='group',
+                                   merge='unique')
             dataset.obsm = datasets[0].obsm
             adata.var.index = adata.var.index.str.replace('/', '_')
             adata.var_names_make_unique()
@@ -50,9 +57,11 @@ def configure_app(app, list_of_dataset_paths, spatial_directories, marker_paths)
         for i in range(len(spatial_directories)):
             spatial_directory = spatial_directories[i]
             if spatial_directory != '':
-                adata = anndata_dataset.get_data(get_fs(dataset_ids[i]), dataset_ids[i])
+                adata = anndata_dataset.get_data(get_fs(dataset_ids[i]),
+                                                 dataset_ids[i])
                 if not add_spatial(adata, spatial_directory):
-                    print('No spatial data found in {}'.format(spatial_directory))
+                    print(
+                        'No spatial data found in {}'.format(spatial_directory))
 
     if marker_paths is not None and len(marker_paths) > 0:
         markers = get_markers(marker_paths)
@@ -68,14 +77,15 @@ def create_app():
     from cirrocumulus.api import cirro_blueprint
     from flask_compress import Compress
     from flask import Flask, send_from_directory
-    os.environ['WERKZEUG_RUN_MAIN'] = 'true'
     app = Flask(__name__, static_folder='client', static_url_path='')
     app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0
     app.register_blueprint(cirro_blueprint, url_prefix='/api')
 
     @app.route('/')
     def root():
-        return send_from_directory(os.path.abspath(os.path.join(app.root_path, "client")), "index.html")
+        return send_from_directory(
+            os.path.abspath(os.path.join(app.root_path, "client")),
+            "index.html")
 
     Compress(app)
     return app
@@ -87,7 +97,8 @@ def main(argsv):
     parser = argparse.ArgumentParser(description='Run cirrocumulus')
     parser.add_argument('dataset',
                         help='Path to dataset in h5ad, loom, Seurat, TileDB, zarr, or STAR-Fusion format. Separate multiple datasets with '
-                             'a comma instead of a space in order to join datasets by cell id', nargs='+')
+                             'a comma instead of a space in order to join datasets by cell id',
+                        nargs='+')
     parser.add_argument('--spatial', help=SPATIAL_HELP, nargs='*')
     parser.add_argument('--markers',
                         help='Path to JSON file that maps name to features. For example {"a":["gene1", "gene2"], "b":["gene3"]}',
@@ -96,15 +107,21 @@ def main(argsv):
                         help='Host IP address')  # set to 0.0.0.0 to make it accessible from other computers WITHOUT SECURITY.
 
     parser.add_argument('--port', help='Server port', default=5000, type=int)
-    parser.add_argument('--no-open', dest='no_open', help='Do not open your web browser', action='store_true')
-    parser.add_argument('--results', help='URL to save user computed results (e.g. differential expression)')
-    parser.add_argument('--ontology', help='Path to ontology in OBO format for annotation')
+    parser.add_argument('--no-open', dest='no_open',
+                        help='Do not open your web browser',
+                        action='store_true')
+    parser.add_argument('--results',
+                        help='URL to save user computed results (e.g. differential expression)')
+    parser.add_argument('--ontology',
+                        help='Path to ontology in OBO format for annotation')
     args = parser.parse_args(argsv)
     if args.results is not None:
         os.environ[CIRRO_JOB_RESULTS] = args.results
     else:
-        os.environ[CIRRO_JOB_RESULTS] = os.path.join(os.path.dirname(args.dataset[0].rstrip('/')), 'results')
-    get_fs(os.environ[CIRRO_JOB_RESULTS]).makedirs(os.environ[CIRRO_JOB_RESULTS], exist_ok=True)
+        os.environ[CIRRO_JOB_RESULTS] = os.path.join(
+            os.path.dirname(args.dataset[0].rstrip('/')), 'results')
+    get_fs(os.environ[CIRRO_JOB_RESULTS]).makedirs(
+        os.environ[CIRRO_JOB_RESULTS], exist_ok=True)
     if args.ontology is not None:
         os.environ[CIRRO_CELL_ONTOLOGY] = args.ontology
     app = create_app()
