@@ -25,5 +25,9 @@ class ZarrDataset(AbstractBackedDataset):
 
     def get_schema(self, filesystem, path):
         g = zarr.open_group(filesystem.get_mapper(path), mode='r')
-        return json.loads(str(g['uns']['cirro-schema'][...])) if 'cirro-schema' in g['uns'] else dataset_schema(g,
-                                                                                                                n_features=0)
+        if 'cirro-schema' in g['uns']:
+            s = str(g['uns']['cirro-schema'][()])
+            # hack to fix encoding error in old version
+            s = s.replace('"id"›', '"id":')
+            return json.loads(s)
+        return dataset_schema(g, n_features=0)
