@@ -5,8 +5,7 @@ import {
     getCategoryValue,
     getFeatureSets,
     NATSORT,
-    SERVER_CAPABILITY_FEATURE_SETS,
-    splitSearchTokens
+    SERVER_CAPABILITY_FEATURE_SETS
 } from "./util";
 import NumberIcon from "./NumberIcon";
 import {InputLabel, Switch, Typography} from '@mui/material';
@@ -16,13 +15,13 @@ import Link from '@mui/material/Link';
 import Tooltip from '@mui/material/Tooltip';
 import ArrowDropDownIcon from '@mui/icons-material/ArrowDropDown';
 import FontDownloadRoundedIcon from '@mui/icons-material/FontDownloadRounded';
-import {findIndex, isArray, isObject} from 'lodash';
+import {findIndex, groupBy, isArray, isObject} from 'lodash';
 import memoize from "memoize-one";
 import React, {useState} from 'react';
 import withStyles from '@mui/styles/withStyles';
 import {
     deleteFeatureSet,
-    deleteView,
+    deleteLink,
     downloadSelectedIds,
     getDatasetFilterNames,
     getEmbeddingKey,
@@ -63,7 +62,7 @@ export const getAnnotationOptions = memoize(
         const options = [];
         obs.forEach(item => {
             options.push({
-                group: 'Continuous', text: item, id: item, icon: <NumberIcon style={{
+                type: FEATURE_TYPE.OBS, text: item, id: item, icon: <NumberIcon style={{
                     marginRight: 2,
                     fontSize: '0.9rem'
                 }}/>
@@ -71,7 +70,7 @@ export const getAnnotationOptions = memoize(
         });
         obsCat.forEach(item => {
             options.push({
-                group: 'Categorical', text: item, id: item, icon: <FontDownloadRoundedIcon style={{
+                type: FEATURE_TYPE.OBS_CAT, text: item, id: item, icon: <FontDownloadRoundedIcon style={{
                     marginRight: 2,
                     fontSize: '0.9rem'
                 }}/>
@@ -106,10 +105,25 @@ const getEmbeddingOptions = memoize(
 const getModulesOptions = memoize((items) => {
         if (items) {
             const options = items.slice();
-            options.sort((item1, item2) => {
-                return NATSORT(item1.id.toLowerCase(), item2.id.toLowerCase());
+            const textField = options.length > 0 && options[0].name != null ? 'name' : 'id';
+
+            options.forEach(item => {
+                if (item.group == null) {
+                    item.group = ''; // set default group
+                }
+                item.text = item[textField];
+                if (item.text.startsWith(item.group + '-')) { // hide group
+                    item.text = item.text.substring(item.group.length + 1);
+                }
             });
-            return options.map(option => option.id);
+            options.sort((item1, item2) => {
+                const g = NATSORT(item1.group.toLowerCase(), item2.group.toLowerCase());
+                if (g !== 0) {
+                    return g;
+                }
+                return NATSORT(item1.text.toLowerCase(), item2.text.toLowerCase());
+            });
+            return options;
         }
         return [];
     }
@@ -139,7 +153,7 @@ const getFeatureSetOptions = memoize((items, categoricalNames) => {
 );
 
 
-function getModuleView(item) {
+function detailsView(item) {
     const keys = Object.keys(item);
     const index = keys.indexOf('id');
     if (index !== -1) {
@@ -169,78 +183,96 @@ function ExplorePanel(props) {
     const [selectedPopupMenuItem, setSelectedPopupMenuItem] = useState(null);
     const [popupAnchorEl, setPopupAnchorEl] = useState(null);
     const [selectedItem, setSelectedItem] = useState({});
+    const {
+        categoricalNames,
+        classes,
+        combineDatasetFilters,
+        dataset,
+        datasetFilter,
+        embeddingLabels,
+        embeddings,
+        embeddingData,
+        handleDialog,
+        handleCombineDatasetFilters,
+        handleEmbeddingLabel,
+        handleDownloadSelectedIds,
+        handleActiveFeature,
+        handleTab,
+        handleDeleteFeatureSet,
+        handleEmbeddings,
+        handleSearchTokens,
+        markers,
+        removeDatasetFilter,
+        searchTokens,
+        selection,
+        serverInfo,
+        tab
+    } = props;
 
     function onDatasetFilterChipDeleted(name) {
-        props.removeDatasetFilter(name);
+        removeDatasetFilter(name);
     }
 
     function onDatasetFilterCleared() {
-        props.removeDatasetFilter(null);
+        removeDatasetFilter(null);
     }
 
-    function handleCombineDatasetFilters(event) {
-        props.handleCombineDatasetFilters(event.target.checked ? 'or' : 'and');
+    function onCombineDatasetFilters(event) {
+        handleCombineDatasetFilters(event.target.checked ? 'or' : 'and');
     }
 
-    function onFeaturesChange(event, value) {
-        props.handleSearchTokens(value, FEATURE_TYPE.X);
+    function onFeaturesChange(event, values) {
+        handleSearchTokens(searchTokens.filter(token => token.type !== FEATURE_TYPE.X).concat(values.map(item => {
+            return {id: item.id, type: FEATURE_TYPE.X};
+        })));
     }
 
-    function onModulesChange(event, value) {
-        props.handleSearchTokens(value, FEATURE_TYPE.MODULE);
+    function onModulesChange(event, values) {
+        handleSearchTokens(searchTokens.filter(token => token.type !== FEATURE_TYPE.MODULE).concat(values.map(item => {
+            return {id: item.id, type: FEATURE_TYPE.MODULE};
+        })));
+    }
+
+    function onObservationsChange(event, values) {
+        handleSearchTokens(searchTokens.filter(token => token.type !== FEATURE_TYPE.OBS && token.type !== FEATURE_TYPE.OBS_CAT).concat(values.map(item => {
+            return {id: item.id, type: item.type};
+        })));
+    }
+
+    function onFeatureSetsChange(event, values) {
+        handleSearchTokens(searchTokens.filter(token => token.type !== FEATURE_TYPE.FEATURE_SET).concat(values.map(item => {
+            return {id: item.id != null ? item.id : item, type: FEATURE_TYPE.FEATURE_SET};
+        })));
     }
 
     function onObservationsIconClick(event, option) {
-        props.handleEmbeddingLabel(option);
+        handleEmbeddingLabel(option);
         event.stopPropagation();
     }
 
-    function onObservationsChange(event, value) {
-        let values = [];
-        value.forEach(val => {
-            if (val.text !== undefined) {
-                values.push(val.text);
-            } else {
-                values.push(val);
-            }
-        });
-        props.handleSearchTokens(values, FEATURE_TYPE.OBS);
-    }
 
     function onSaveFeatureList() {
-        props.handleDialog(SAVE_FEATURE_SET_DIALOG);
+        handleDialog(SAVE_FEATURE_SET_DIALOG);
     }
 
 
     function onEmbeddingsChange(event, value) {
-        const selection = [];
-        const embeddingKeys = props.dataset.embeddings.map(item => getEmbeddingKey(item));
+        const newValue = [];
+        const embeddingKeys = dataset.embeddings.map(item => getEmbeddingKey(item));
         value.forEach(val => {
             const id = val.id !== undefined ? val.id : val;
             const index = embeddingKeys.indexOf(id);
-            let embedding = props.dataset.embeddings[index];
-            selection.push(embedding);
+            let embedding = dataset.embeddings[index];
+            newValue.push(embedding);
         });
-        props.handleEmbeddings(selection);
+        handleEmbeddings(newValue);
     }
 
-    function onFeatureSetsChange(event, value) {
-        let values = [];
-        value.forEach(val => {
-            if (val.id !== undefined) {
-                values.push(val.id);
-            } else {
-                values.push(val);
-            }
-        });
-        props.handleSearchTokens(values, FEATURE_TYPE.FEATURE_SET);
-    }
 
     function onFeatureSetClick(event, option) {
         event.stopPropagation();
         const id = option.id;
         const target = event.target.closest(".MuiFormControl-root");
-        let markers = props.markers;
         let newFeatureSet = null;
         for (let i = 0; i < markers.length; i++) {
             if (markers[i].id === id) {
@@ -258,11 +290,10 @@ function ExplorePanel(props) {
     function onModulesClick(event, option) {
         event.stopPropagation();
         const target = event.target.closest(".MuiFormControl-root");
-
-        const modules = props.dataset.modules;
+        const modules = dataset.modules;
         let selectedItem;
         for (let i = 0, n = modules.length; i < n; i++) {
-            if (modules[i].id == option) {
+            if (modules[i].id === option.id) {
                 selectedItem = modules[i];
                 break;
             }
@@ -296,11 +327,10 @@ function ExplorePanel(props) {
 
     function onDeleteFeatureSet(event) {
         event.stopPropagation();
-        let searchTokens = props.searchTokens;
         const featureSetId = selectedItem.value.id;
-        let value = searchTokens.filter(token => token.type === FEATURE_TYPE.FEATURE_SET && token.value.id !== featureSetId);
-        props.handleSearchTokens(value, FEATURE_TYPE.FEATURE_SET);
-        props.handleDeleteFeatureSet(featureSetId);
+        const values = searchTokens.filter(token => token.type === FEATURE_TYPE.FEATURE_SET && token.id !== featureSetId);
+        handleSearchTokens(searchTokens.filter(token => token.type !== FEATURE_TYPE.FEATURE_SET).concat(values));
+        handleDeleteFeatureSet(featureSetId);
         setSelectedItem({});
         setPopupAnchorEl(null);
     }
@@ -312,25 +342,25 @@ function ExplorePanel(props) {
     function onFeatureCopy(event) {
         event.preventDefault();
         event.stopPropagation();
-        let searchTokens = props.searchTokens;
-        copyToClipboard(searchTokens.filter(token => token.type === FEATURE_TYPE.X).map(item => item.value).join('\n'));
+        copyToClipboard(searchTokens.filter(token => token.type === FEATURE_TYPE.X).map(item => item.id).join('\n'));
+
     }
 
     function onDownloadSelectedIds(event) {
         event.preventDefault();
-        props.handleDownloadSelectedIds();
+        handleDownloadSelectedIds();
     }
 
     function onFeatureClick(event, option) {
         event.stopPropagation();
-        const value = option.text !== undefined ? option.text : option;
-        let galleryTraces = props.embeddingData.filter(traceInfo => traceInfo.active);
+        const value = option.id !== undefined ? option.id : option;
+        let galleryTraces = embeddingData.filter(trace => trace.active);
         for (let i = 0; i < galleryTraces.length; i++) {
             if (galleryTraces[i].name === value) {
                 if (props.tab !== 'embedding') {
-                    props.handleTab('embedding');
+                    handleTab('embedding');
                 }
-                props.handleActiveFeature({
+                handleActiveFeature({
                     name: galleryTraces[i].name,
                     type: galleryTraces[i].featureType,
                     embeddingKey: getTraceKey(galleryTraces[i])
@@ -340,25 +370,14 @@ function ExplorePanel(props) {
         }
     }
 
-    const {
-        categoricalNames,
-        classes,
-        combineDatasetFilters,
-        dataset,
-        datasetFilter,
-        embeddingLabels,
-        embeddings,
-        markers,
-        searchTokens,
-        selection,
-        serverInfo,
-        tab
-    } = props;
 
     const datasetFilterKeys = getDatasetFilterNames(datasetFilter);
     datasetFilterKeys.sort(NATSORT);
-    const splitTokens = splitSearchTokens(searchTokens);
-    const featureSets = getFeatureSets(markers, splitTokens.featureSets);
+    const groupedSearchTokens = groupBy(searchTokens, 'type');
+    const obsCatSearchTokens = (groupedSearchTokens[FEATURE_TYPE.OBS_CAT] || []).map(item => item.id);
+    const xSearchTokens = groupedSearchTokens[FEATURE_TYPE.X] || [];
+    const featureSets = getFeatureSets(markers, groupedSearchTokens[FEATURE_TYPE.FEATURE_SET] || []);
+    const moduleTokens = groupedSearchTokens[FEATURE_TYPE.MODULE] || [];
     const featureOptions = dataset.features;
     const moduleOptions = getModulesOptions(dataset.modules);
     const obsCat = dataset.obsCat;
@@ -367,6 +386,7 @@ function ExplorePanel(props) {
     const featureSetOptions = getFeatureSetOptions(markers, categoricalNames);
     const embeddingOptions = getEmbeddingOptions(dataset.embeddings);
     const selectedEmbeddings = getEmbeddingOptions(embeddings);
+
     return <>
 
         {'feature set view' === selectedPopupMenuItem && <Dialog
@@ -410,16 +430,7 @@ function ExplorePanel(props) {
         >
             <DialogTitle id="view-dialog-title">{selectedItem ? selectedItem.value.id : ''}</DialogTitle>
             <DialogContent>
-
-                {getModuleView(selectedItem.value)}
-                {/*<DataGrid*/}
-                {/*    rows={rows}*/}
-                {/*    columns={columns}*/}
-                {/*    pageSize={5}*/}
-                {/*    rowsPerPageOptions={[5]}*/}
-                {/*    checkboxSelection*/}
-                {/*    disableSelectionOnClick*/}
-                {/*/>*/}
+                {detailsView(selectedItem.value)}
             </DialogContent>
         </Dialog>}
 
@@ -432,20 +443,16 @@ function ExplorePanel(props) {
             <MenuItem onClick={onViewModule}>View</MenuItem>
         </Menu>}
         <div style={tab === 'embedding' || tab === 'distribution' || tab === 'composition' ? null : {display: 'none'}}>
-
             <Typography gutterBottom={false} component={"h1"}
                         style={{textTransform: 'uppercase', letterSpacing: '0.1em'}}>Explore</Typography>
-            {tab === 'embedding' && embeddingOptions.length > 0 &&
+            {embeddingOptions.length > 0 &&
             <FormControl sx={{display: 'block'}}>
                 <AutocompleteVirtualized label={"Embeddings"}
                                          testId={'embeddings-input'}
                                          options={embeddingOptions}
-                                         getChipTitle={(option) => {
-                                             return option.text;
-                                         }}
+                                         getChipText={(option) => option.text}
                                          getOptionLabel={(option) => option.text}
                                          value={selectedEmbeddings}
-                                         getChipText={(option) => option.text}
                                          getOptionSelected={(option, value) => findIndex(selectedEmbeddings, item => item.id === option.id) !== -1}
                                          onChange={onEmbeddingsChange}
                 />
@@ -455,9 +462,12 @@ function ExplorePanel(props) {
                                          label={"Genes/Features"}
                                          testId={'genes-input'}
                                          options={featureOptions}
-                                         value={splitTokens.X}
-                                         getOptionLabel={(option) => option}
+                                         getChipText={(option) => option.id}
+                                         value={xSearchTokens}
+                                         getOptionSelected={(option, value) => option.id === value.id}
+                                         groupBy={(option) => option.group}
                                          onChange={onFeaturesChange}
+                                         getOptionLabel={(option) => option.text}
                                          helperText={"Enter or paste list"}
 
                 />
@@ -467,23 +477,35 @@ function ExplorePanel(props) {
                         marginRight: 4,
                         fontSize: '0.75rem',
                         transform: 'translateY(-50px)',
-                        display: splitTokens.X.length === 0 ? 'none' : ''
+                        display: xSearchTokens.length === 0 ? 'none' : ''
                     }}
                     onClick={onFeatureCopy}>Copy</Link></div>
 
             </FormControl>}
+            {/*{layerOptions.length > 0 &&*/}
+            {/*<FormControl sx={{display: 'block'}}>*/}
+            {/*    <AutocompleteVirtualized label={"Layers"}*/}
+            {/*                             testId={'layers-input'}*/}
+            {/*                             options={layerOptions}*/}
+            {/*                             value={layers}*/}
+            {/*                             getOptionLabel={(option) => option}*/}
+            {/*                             onChange={onLayersChange}*/}
+            {/*    />*/}
+            {/*</FormControl>}*/}
             {annotationOptions.length > 0 && <FormControl sx={{display: 'block'}}>
                 <AutocompleteVirtualized label={"Cell Metadata"}
                                          testId={'cell-meta-input'}
                                          options={annotationOptions}
-                                         value={searchTokens.filter(token => token.type === FEATURE_TYPE.OBS_CAT || token.type === FEATURE_TYPE.OBS).map(token => token.value)}
+                                         getChipText={(option) => option.id}
+                                         value={searchTokens.filter(token => token.type === FEATURE_TYPE.OBS_CAT || token.type === FEATURE_TYPE.OBS)}
                                          onChipClick={onFeatureClick}
+                                         getOptionSelected={(option, value) => option.id === value.id}
                                          getOptionLabel={(option) => option.text}
                                          getChipIcon={(option) => {
-                                             return splitTokens.obsCat.indexOf(option) !== -1 ?
+                                             return obsCatSearchTokens.indexOf(option.id) !== -1 ?
                                                  <FontDownloadRoundedIcon
                                                      onClick={(event) => {
-                                                         onObservationsIconClick(event, option);
+                                                         onObservationsIconClick(event, option.id);
                                                      }}
                                                      title={"Toggle Show/Hide Labels"}
                                                      style={{
@@ -492,19 +514,22 @@ function ExplorePanel(props) {
                                                          marginRight: 0,
                                                          marginBottom: 0
                                                      }}
-                                                     className={"MuiChip-deleteIcon MuiChip-deleteIconSmall" + (embeddingLabels.indexOf(option) !== -1 ? ' cirro-active' : '')}/> : null;
+                                                     className={"MuiChip-deleteIcon MuiChip-deleteIconSmall" + (embeddingLabels.indexOf(option.id) !== -1 ? ' cirro-active' : '')}/> : null;
                                          }}
-                                         getOptionSelected={(option, value) => option.id === value}
                                          onChange={onObservationsChange}/>
             </FormControl>}
-
             {moduleOptions.length > 0 && <FormControl sx={{display: 'block'}}>
                 <AutocompleteVirtualized
                     label={"Modules"}
                     testId={'modules-input'}
                     options={moduleOptions}
-                    value={splitTokens.modules}
+                    getChipText={(option) => option.id}
+                    value={moduleTokens}
+                    getOptionSelected={(option, value) => option.id === value.id}
+                    groupBy={(option) => option.group}
+                    selectGroup={true}
                     onChange={onModulesChange}
+                    getOptionLabel={(option) => option.text}
                     onChipClick={onModulesClick}
                     getChipIcon={(option) => {
                         return <ArrowDropDownIcon onClick={(event) => {
@@ -518,9 +543,12 @@ function ExplorePanel(props) {
                                          testId={'sets-input'}
                                          options={featureSetOptions}
                                          value={featureSets}
+                                         getChipText={(option) => option.name}
                                          getChipTitle={(option) => {
                                              return option.category + ', ' + option.name;
                                          }}
+                                         getOptionLabel={(option) => option.text}
+                                         selectGroup={true}
                                          onChipClick={onFeatureSetClick}
                                          getChipIcon={(option) => {
                                              return <ArrowDropDownIcon onClick={(event) => {
@@ -530,7 +558,7 @@ function ExplorePanel(props) {
                                          groupBy={(option) => option.group}
                                          onChange={onFeatureSetsChange}
                                          getOptionSelected={(option, value) => option.id === value.id}
-                                         getChipText={option => option.name}
+
                 />
                 {serverInfo.capabilities.has(SERVER_CAPABILITY_FEATURE_SETS) && <div>
                     <Tooltip title={"Save Current Genes/Features"}>
@@ -539,29 +567,30 @@ function ExplorePanel(props) {
                                 float: 'right',
                                 fontSize: '0.75rem',
                                 marginRight: 4,
-                                display: splitTokens.X.length === 0 ? 'none' : ''
+                                display: xSearchTokens.length === 0 ? 'none' : ''
                             }}
                             onClick={onSaveFeatureList}>Save</Link></Tooltip></div>}
             </FormControl>}
         </div>
-        <div style={{maxHeight: 500}}>
+        <div
+            style={tab === 'embedding' || tab === 'distribution' || tab === 'composition' ? {maxHeight: 500} : {display: 'none'}}>
             <Divider inset="true"/>
             <Typography gutterBottom={false} component={"h1"}
                         style={{textTransform: 'uppercase'}}>Filters</Typography>
             <Grid alignContent={"flex-start"} container alignItems="center"
                   spacing={0}>
                 <Grid item><InputLabel shrink={true}>Combine</InputLabel></Grid>
-                <Grid item>AND</Grid>
+                <Grid item><InputLabel>AND</InputLabel></Grid>
                 <Grid item>
                     <Switch
                         size="small"
                         checked={combineDatasetFilters === 'or'}
-                        onChange={handleCombineDatasetFilters}
+                        onChange={onCombineDatasetFilters}
                     />
                 </Grid>
-                <Grid item>OR</Grid>
+                <Grid item><InputLabel>OR</InputLabel></Grid>
             </Grid>
-            {datasetFilterKeys.length > 0 && selection.size > 0 &&
+            {datasetFilterKeys.length > 0 && selection != null &&
             <>
                 <div style={{marginBottom: 2}}>
                     {intFormat(selection.size) + " / " + intFormat(dataset.shape[0]) + ": "}
@@ -619,7 +648,7 @@ const mapStateToProps = state => {
         };
     }
 ;
-const mapDispatchToProps = (dispatch, ownProps) => {
+const mapDispatchToProps = (dispatch) => {
     return {
         handleDialog: (value) => {
             dispatch(setDialog(value));
@@ -645,11 +674,11 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         handleEmbeddingLabel: value => {
             dispatch(toggleEmbeddingLabel(value));
         },
-        handleSearchTokens: (value, type) => {
-            dispatch(setSearchTokens(value == null ? [] : value, type));
+        handleSearchTokens: value => {
+            dispatch(setSearchTokens(value));
         },
         handleDeleteView: value => {
-            dispatch(deleteView(value));
+            dispatch(deleteLink(value));
         },
         handleDeleteFeatureSet: value => {
             dispatch(deleteFeatureSet(value));
