@@ -15,13 +15,13 @@ function JobResultPanel(props) {
 
 const mapStateToProps = state => {
         return {
-            jobResultId: state.jobResult,
+            jobResultId: state.jobResultId,
             jobResults: state.jobResults,
             tab: state.tab
         };
     }
 ;
-const mapDispatchToProps = (dispatch, ownProps) => {
+const mapDispatchToProps = (dispatch) => {
         return {};
     }
 ;
