@@ -4,7 +4,6 @@ import React, {useEffect, useRef, useState} from 'react';
 import {Color, Vector3, Vector4} from 'three';
 import {getEmbeddingKey} from './actions';
 import ChartToolbar from './ChartToolbar';
-import {saveImage} from './ChartUtil';
 import {numberFormat2f} from './formatters';
 import {
     createScatterPlot,
@@ -14,6 +13,8 @@ import {
     updateScatterChart
 } from './ThreeUtil';
 import {indexSort, isPointInside} from './util';
+import {saveImage} from './ChartUtil';
+import CirroTooltip, {SCATTER_TRANSITION} from './CirroTooltip';
 
 function clamp(x, min_v, max_v) {
     return Math.min(Math.max(x, min_v), max_v);
@@ -100,84 +101,157 @@ function ScatterChartThree(props) {
     const lastHoverIndexRef = useRef();
     const [forceUpdate, setForceUpdate] = useState(false);
     const previousChartSizeRef = useRef();
+    const previousCameraPosition = useRef({x: -1, y: -1, z: -1});
+    const [tip, setTip] = useState({html: ''});
 
-    function getSelectedIndex(point) {
-        const trace = props.trace;
-        const positions = trace.positions;
-        const camera = scatterPlotRef.current.camera;
-        const widthHalf = props.chartSize.width / 2;
-        const heightHalf = props.chartSize.height / 2;
-        const pos = new Vector3();
-        let selectedIndex = -1;
-        const tolerance = 2;
-        if (lastHoverIndexRef.current !== -1) {
-            pos.x = positions[lastHoverIndexRef.current * 3];
-            pos.y = positions[lastHoverIndexRef.current * 3 + 1];
-            pos.z = positions[lastHoverIndexRef.current * 3 + 2];
-            pos.project(camera);
-            pos.x = (pos.x * widthHalf) + widthHalf;
-            pos.y = -(pos.y * heightHalf) + heightHalf;
-            if (Math.abs(pos.x - point.x) <= tolerance && Math.abs(pos.y - point.y) <= tolerance) {
-                selectedIndex = lastHoverIndexRef.current;
-            }
+    const {
+        cachedData,
+        categoricalNames,
+        chartOptions,
+        chartSize,
+        classes,
+        handleClick,
+        markerOpacity,
+        obsCat,
+        onCamera,
+        onGallery,
+        onSelected,
+        pointSize,
+        selection,
+        setChartOptions,
+        trace,
+        unselectedMarkerOpacity,
+        unselectedPointSize
+    } = props;
+
+    const darkMode = chartOptions.darkMode;
+    const showAxis = chartOptions.showAxis;
+    const showFog = chartOptions.showFog;
+
+    useEffect(() => {
+        function webglcontextlost(e) {
+            console.log('lost webgl context');
+            e.preventDefault();
         }
 
-        if (selectedIndex === -1) {
-            // TODO get all hover points
-            for (let i = 0, k = 0, npoints = trace.x.length; i < npoints; i++, k += 3) {
-                pos.x = positions[k];
-                pos.y = positions[k + 1];
-                pos.z = positions[k + 2];
+        function webglcontextrestored(e) {
+            console.log('restored webgl context');
+            e.preventDefault();
+            setForceUpdate(c => !c);
+        }
+
+        if (scatterPlotRef.current == null) {
+            const dragmode = chartOptions.dragmode;
+            scatterPlotRef.current = createScatterPlot(containerElementRef.current, window.ApplePaySession, true);
+            if (dragmode === 'pan') {
+                scatterPlotRef.current.setInteractionMode('PAN');
+            } else if (dragmode === 'select') {
+                scatterPlotRef.current.setInteractionMode('SELECT');
+                scatterPlotRef.current.rectangleSelector.setSelectionMode('BOX');
+            } else if (dragmode === 'lasso') {
+                scatterPlotRef.current.setInteractionMode('SELECT');
+                scatterPlotRef.current.rectangleSelector.setSelectionMode('LASSO');
+            }
+
+            const canvas = containerElementRef.current.querySelector('canvas');
+            canvas.style.outline = '0px';
+            canvas.addEventListener('webglcontextlost', webglcontextlost);
+            canvas.addEventListener('webglcontextrestored', webglcontextrestored);
+        }
+        if (chartOptions.camera) {
+            scatterPlotRef.current.updateFromCameraDef(chartOptions.camera);
+            chartOptions.camera = null;
+        }
+        chartOptions.scatterPlot = scatterPlotRef.current;
+        return () => {
+            if (containerElementRef.current) {
+                const canvas = containerElementRef.current.querySelector('canvas');
+                canvas.removeEventListener('webglcontextlost', webglcontextlost);
+                canvas.removeEventListener('webglcontextrestored', webglcontextrestored);
+            }
+        };
+    }, [scatterPlotRef, containerElementRef, chartOptions]);
+
+
+    useEffect(() => {
+
+        function handleCamera(eventName) {
+            onCamera(eventName, scatterPlotRef.current.getCameraDef());
+        }
+
+        function getSelectedIndex(point) {
+            const positions = trace.positions;
+            const camera = scatterPlotRef.current.camera;
+            const widthHalf = chartSize.width / 2;
+            const heightHalf = chartSize.height / 2;
+            const pos = new Vector3();
+            let selectedIndex = -1;
+            const tolerance = 2;
+            if (lastHoverIndexRef.current !== -1) {
+                pos.x = positions[lastHoverIndexRef.current * 3];
+                pos.y = positions[lastHoverIndexRef.current * 3 + 1];
+                pos.z = positions[lastHoverIndexRef.current * 3 + 2];
                 pos.project(camera);
                 pos.x = (pos.x * widthHalf) + widthHalf;
                 pos.y = -(pos.y * heightHalf) + heightHalf;
                 if (Math.abs(pos.x - point.x) <= tolerance && Math.abs(pos.y - point.y) <= tolerance) {
-                    selectedIndex = i;
-                    break;
+                    selectedIndex = lastHoverIndexRef.current;
                 }
             }
-        }
-        lastHoverIndexRef.current = selectedIndex;
-        return selectedIndex;
-    }
 
-    useEffect(() => {
-        init();
-        if (previousChartSizeRef.current !== props.chartSize) {
-            scatterPlotRef.current.resize();
+            if (selectedIndex === -1) {
+                // TODO get all hover points
+                for (let i = 0, k = 0, npoints = trace.values.length; i < npoints; i++, k += 3) {
+                    pos.x = positions[k];
+                    pos.y = positions[k + 1];
+                    pos.z = positions[k + 2];
+                    pos.project(camera);
+                    pos.x = (pos.x * widthHalf) + widthHalf;
+                    pos.y = -(pos.y * heightHalf) + heightHalf;
+                    if (Math.abs(pos.x - point.x) <= tolerance && Math.abs(pos.y - point.y) <= tolerance) {
+                        selectedIndex = i;
+                        break;
+                    }
+                }
+            }
+            lastHoverIndexRef.current = selectedIndex;
+            return selectedIndex;
         }
-        draw();
-        setAxesColors(scatterPlotRef.current, props.chartOptions.darkMode);
-        props.chartOptions.scatterPlot = scatterPlotRef.current;
-        if (props.chartOptions.camera) {
-            scatterPlotRef.current.updateFromCameraDef(props.chartOptions.camera);
-            props.chartOptions.camera = null;
-        }
-        previousChartSizeRef.current = props.chartSize;
+
+
+        let singleClickTimer;
+        let clickCount = 0;
         scatterPlotRef.current.clickCallback = (point, append) => {
-            if (!props.trace.continuous) {
-                const selectedIndex = getSelectedIndex(point);
-                if (selectedIndex !== -1) {
-                    props.handleClick({
-                        name: props.trace.name,
-                        value: props.trace.values[selectedIndex],
-                        shiftKey: false,
-                        metaKey: append
-                    });
+            if (!trace.continuous) {
+                clickCount++;
+                if (clickCount === 1) {
+                    singleClickTimer = setTimeout(() => {
+                        clickCount = 0; // reset after timeout
+                    }, 400);
+                } else if (clickCount === 2) {
+                    clearTimeout(singleClickTimer);
+                    clickCount = 0;
+                    const selectedIndex = getSelectedIndex(point);
+                    if (selectedIndex !== -1) {
+                        handleClick({
+                            name: trace.name,
+                            value: trace.values[selectedIndex],
+                            shiftKey: false,
+                            metaKey: append
+                        });
+                    }
+                    lastHoverIndexRef.current = selectedIndex;
                 }
-                lastHoverIndexRef.current = selectedIndex;
             }
-
         };
-        scatterPlotRef.current.hoverCallback = (point) => {
+        scatterPlotRef.current.hoverCallback = (point, event) => {
             if (point == null) {
-                props.setTooltip('');
+                setTip({html: ''});
             } else {
                 const selectedIndex = getSelectedIndex(point);
                 if (selectedIndex !== -1) {
-                    const {trace} = props;
                     let value = trace.values[selectedIndex];
-                    let categoryObject = props.categoricalNames[trace.name] || {};
+                    let categoryObject = categoricalNames[trace.name] || {};
                     let renamedValue = categoryObject[value];
                     if (renamedValue != null && renamedValue.newValue != null) {
                         value = renamedValue.newValue;
@@ -189,22 +263,23 @@ function ScatterChartThree(props) {
                             value = value.substring(0, value.lastIndexOf('.'));
                         }
                     }
-                    props.setTooltip('' + value);
+                    setTip({html: '' + value, clientX: event.clientX, clientY: event.clientY});
+                    // updateTooltipText(tip, '' + value, event);
                 } else {
-                    props.setTooltip('');
+                    setTip({html: ''});
+                    // updateTooltipText(tip, '', event);
                 }
             }
         };
         scatterPlotRef.current.lassoCallback = (points, appendToSelection) => {
-            const trace = props.trace;
             const positions = trace.positions;
             const camera = scatterPlotRef.current.camera;
-            const widthHalf = props.chartSize.width / 2;
-            const heightHalf = props.chartSize.height / 2;
+            const widthHalf = chartSize.width / 2;
+            const heightHalf = chartSize.height / 2;
             const pos = new Vector3();
             const selectedIndices = new Set();
 
-            for (let i = 0, k = 0, npoints = trace.x.length; i < npoints; i++, k += 3) {
+            for (let i = 0, k = 0, npoints = trace.values.length; i < npoints; i++, k += 3) {
                 pos.x = positions[k];
                 pos.y = positions[k + 1];
                 pos.z = positions[k + 2];
@@ -216,9 +291,9 @@ function ScatterChartThree(props) {
                 }
             }
             if (selectedIndices.size === 0) {
-                props.onSelected({name: getEmbeddingKey(trace.embedding)});
+                onSelected({name: getEmbeddingKey(trace.embedding)});
             } else {
-                props.onSelected({
+                onSelected({
                     name: getEmbeddingKey(trace.embedding),
                     clear: !appendToSelection,
                     value: {basis: trace.embedding, indices: selectedIndices}
@@ -229,15 +304,14 @@ function ScatterChartThree(props) {
             if (scatterPlotRef.current.interactionMode === 'PAN') {
                 return;
             }
-            const trace = props.trace;
             const positions = trace.positions;
             const camera = scatterPlotRef.current.camera;
-            const widthHalf = props.chartSize.width / 2;
-            const heightHalf = props.chartSize.height / 2;
+            const widthHalf = chartSize.width / 2;
+            const heightHalf = chartSize.height / 2;
             const pos = new Vector3();
             const selectedIndices = new Set();
 
-            for (let i = 0, k = 0, npoints = trace.x.length; i < npoints; i++, k += 3) {
+            for (let i = 0, k = 0, npoints = trace.values.length; i < npoints; i++, k += 3) {
                 pos.x = positions[k];
                 pos.y = positions[k + 1];
                 pos.z = positions[k + 2];
@@ -250,24 +324,52 @@ function ScatterChartThree(props) {
             }
 
             if (selectedIndices.size === 0) {
-                props.onSelected({name: getEmbeddingKey(trace.embedding)});
+                onSelected({name: getEmbeddingKey(trace.embedding)});
             } else {
-                props.onSelected({
+                onSelected({
                     name: getEmbeddingKey(trace.embedding),
                     clear: !appendToSelection,
                     value: {basis: trace.embedding, indices: selectedIndices}
                 });
             }
         };
-        scatterPlotRef.current.cameraCallback = (eventName) => {
-            if (scatterPlotRef.current.interactionMode === 'PAN' && props.trace.dimensions === 3) {
+        scatterPlotRef.current.cameraCallback = (eventName, position, target) => {
+            if (scatterPlotRef.current.interactionMode === 'PAN' && trace.dimensions === 3) {
                 // repaint gallery charts with same embedding
-                if (eventName === 'end') {
-                    cameraCallback(eventName);
+                if (eventName === 'end' && (previousCameraPosition.current.x != position.x || previousCameraPosition.current.y != position.y || previousCameraPosition.current.z != position.z)) {
+                    previousCameraPosition.current = {x: position.x, y: position.y, z: position.z};
+                    handleCamera(eventName);
                 }
             }
         };
-    });
+        return () => {
+            scatterPlotRef.current.clickCallback = null;
+            scatterPlotRef.current.hoverCallback = null;
+            scatterPlotRef.current.lassoCallback = null;
+            scatterPlotRef.current.boxCallback = null;
+            scatterPlotRef.current.cameraCallback = null;
+        };
+    }, [scatterPlotRef, categoricalNames, chartSize, trace]); // onSelected, handleClick, onCamera
+
+    useEffect(() => {
+        setAxesColors(scatterPlotRef.current, darkMode);
+        const axes = scatterPlotRef.current.scene.getObjectByName('axes');
+        if (axes) {
+            axes.visible = showAxis;
+        }
+        getVisualizer(scatterPlotRef.current, POINT_VISUALIZER_ID).styles.fog.enabled = showFog;
+    }, [scatterPlotRef, darkMode, showAxis, showFog]);
+
+
+    useEffect(() => {
+        if (previousChartSizeRef.current !== chartSize) {
+            scatterPlotRef.current.resize();
+        }
+        previousChartSizeRef.current = chartSize;
+        updateScatterChart(scatterPlotRef.current, trace, selection, markerOpacity, unselectedMarkerOpacity, pointSize, unselectedPointSize,
+            categoricalNames, chartOptions, obsCat, cachedData);
+    }, [scatterPlotRef, trace, selection, markerOpacity, unselectedMarkerOpacity, pointSize, unselectedPointSize,
+        categoricalNames, chartOptions, obsCat, cachedData, chartSize, forceUpdate]);
 
     useEffect(() => {
         return () => {
@@ -276,7 +378,7 @@ function ScatterChartThree(props) {
     }, []);
 
     function calculatePointSize(trace) {
-        const n = trace.x.length;
+        const n = trace.values.length;
         const SCALE = 200;
         const LOG_BASE = 8;
         const DIVISOR = 1.5;
@@ -286,25 +388,14 @@ function ScatterChartThree(props) {
     }
 
     function drawContext(context, chartSize, format) {
-        const {
-            obsCat,
-            cachedData,
-            trace,
-            markerOpacity,
-            unselectedMarkerOpacity,
-            selection,
-            categoricalNames,
-            chartOptions
-        } = props;
-        const scatterPlot = scatterPlotRef.current;
-        const pointSize = calculatePointSize(trace);
-        const scaleFactor = props.pointSize;
         const PI2 = 2 * Math.PI;
-        const colors = trace.colors;
-        const positions = trace.positions;
-        const camera = scatterPlot.camera;
         const width = chartSize.width;
         const height = chartSize.height;
+        const scatterPlot = scatterPlotRef.current;
+        const colors = trace.colors;
+        const positions = trace.positions;
+
+
         if (chartOptions.darkMode) {
             context.fillStyle = 'black';
             context.fillRect(0, 0, width, height);
@@ -312,40 +403,18 @@ function ScatterChartThree(props) {
         const widthHalf = width / 2;
         const heightHalf = height / 2;
         const colorScaleConverter = scaleLinear().domain([0, 1]).range([0, 255]);
-        const npoints = trace.x.length;
+        const npoints = trace.values.length;
         const is3d = trace.dimensions === 3;
-        let outputPointSize;
-        let fog = scatterPlot.scene.fog;
-        let spriteVisualizer = getVisualizer(scatterPlot, POINT_VISUALIZER_ID);
-        // const zoomFactor = getScaleFactor(props.chartSize);
-        // const zoomFactorSpecified = false;
-
-        if (!is3d) {
-            const PI = 3.1415926535897932384626433832795;
-            const minScale = 0.1;  // minimum scaling factor
-            const outSpeed = 2.0;  // shrink speed when zooming out
-            const outNorm = (1. - minScale) / Math.atan(outSpeed);
-            const maxScale = 15.0;  // maximum scaling factor
-            const inSpeed = 0.02;  // enlarge speed when zooming in
-            const zoomOffset = 0.3;  // offset zoom pivot
-            let m = camera.projectionMatrix.elements[0];
-            // if (zoomFactorSpecified) {
-            //     m = zoomFactor;
-            // }
-            let zoom = m + zoomOffset;  // zoom pivot
-            let scale = zoom < 1. ? 1. + outNorm * Math.atan(outSpeed * (zoom - 1.)) :
-                1. + 2. / PI * (maxScale - 1.) * Math.atan(inSpeed * (zoom - 1.));
-            outputPointSize = pointSize * scale;
-        }
-        let gl_PointSize = (outputPointSize * scaleFactor);
-        gl_PointSize /= 2;
+        const fog = scatterPlot.scene.fog;
+        const camera = scatterPlot.camera;
+        const spriteVisualizer = getVisualizer(scatterPlot, POINT_VISUALIZER_ID);
+        const scaleFactor = is3d ? calculatePointSize(trace) : 1; // TODO
         const pos = new Vector3();
-        let cameraSpacePos = new Vector4();
-        let object = spriteVisualizer.points;
-        let modelViewMatrix = object.modelViewMatrix.clone();
-        modelViewMatrix.multiplyMatrices(camera.matrixWorldInverse, object.matrixWorld);
-        const showFog = chartOptions.showFog;
-        const isSelectionEmpty = selection.size === 0;
+        const cameraSpacePos = new Vector4();
+        const modelViewMatrix = spriteVisualizer.points.modelViewMatrix.clone();
+        modelViewMatrix.multiplyMatrices(camera.matrixWorldInverse, spriteVisualizer.points.matrixWorld);
+        const showFog = chartOptions.showFog && is3d;
+        const isSelectionEmpty = selection == null;
         let pointOrder;
         if (is3d) {
             pointOrder = new Uint32Array(npoints);
@@ -373,6 +442,7 @@ function ScatterChartThree(props) {
             let g = colors[colorIndex + 1];
             let b = colors[colorIndex + 2];
             let a = isSelected ? markerOpacity : unselectedMarkerOpacity;
+            let outputPointSize;
             if (is3d) {
                 cameraSpacePos.x = positions[positionIndex];
                 cameraSpacePos.y = positions[positionIndex + 1];
@@ -380,22 +450,42 @@ function ScatterChartThree(props) {
                 cameraSpacePos.w = 1;
                 cameraSpacePos.applyMatrix4(modelViewMatrix);
                 outputPointSize = -pointSize / cameraSpacePos.z;
-                gl_PointSize = (outputPointSize * scaleFactor) / 4;
-                if (showFog) {
-                    const fogDepth = pointSize / outputPointSize * 1.2;
-                    const fogFactor = smoothstep(fog.near, fog.far, fogDepth);
-                    r = mix(r, fog.color.r, fogFactor);
-                    g = mix(g, fog.color.g, fogFactor);
-                    b = mix(b, fog.color.b, fogFactor);
-                }
+            } else {  // Create size attenuation (if we're in 2D mode)
+                const PI = 3.1415926535897932384626433832795;
+                const minScale = 0.1;  // minimum scaling factor
+                const outSpeed = 2.0;  // shrink speed when zooming out
+                const outNorm = (1. - minScale) / Math.atan(outSpeed);
+                const maxScale = 15.0;  // maximum scaling factor
+                const inSpeed = 0.02;  // enlarge speed when zooming in
+                const zoomOffset = 0.3;  // offset zoom pivot
+
+                cameraSpacePos.x = positions[positionIndex];
+                cameraSpacePos.y = positions[positionIndex + 1];
+                cameraSpacePos.z = positions[positionIndex + 2];
+                cameraSpacePos.w = 1;
+                cameraSpacePos.applyMatrix4(modelViewMatrix);
+                let m = -cameraSpacePos.z;
+                // if (zoomFactorSpecified) {
+                //     m = zoomFactor;
+                // }
+                const zoom = m + zoomOffset;  // zoom pivot
+                const scale = zoom < 1. ? 1. + outNorm * Math.atan(outSpeed * (zoom - 1.)) :
+                    1. + 2. / PI * (maxScale - 1.) * Math.atan(inSpeed * (zoom - 1.));
+                outputPointSize = pointSize * scale;
             }
+            if (showFog) {
+                const fogDepth = pointSize / outputPointSize * 1.2;
+                const fogFactor = smoothstep(fog.near, fog.far, fogDepth);
+                r = mix(r, fog.color.r, fogFactor);
+                g = mix(g, fog.color.g, fogFactor);
+                b = mix(b, fog.color.b, fogFactor);
+            }
+            const gl_PointSize = (outputPointSize * scaleFactor) / 2;
             pos.x = (pos.x * widthHalf) + widthHalf;
             pos.y = -(pos.y * heightHalf) + heightHalf;
-
             r = Math.round(colorScaleConverter(r));
             g = Math.round(colorScaleConverter(g));
             b = Math.round(colorScaleConverter(b));
-
             context.fillStyle = 'rgba(' + r + ',' + g + ',' + b + ',' + a + ')';
             context.beginPath();
             context.arc(pos.x, pos.y, gl_PointSize, 0, PI2);
@@ -404,14 +494,13 @@ function ScatterChartThree(props) {
         }
         if (obsCat.length > 0) {
             const labelsPositions = getCategoryLabelsPositions(trace.embedding, obsCat, cachedData);
-            let font = format === 'svg' ? 'serif' : 'Roboto Condensed';
+            const font = format === 'svg' ? 'serif' : 'Roboto Condensed';
             context.font = 'bold ' + chartOptions.labelFontSize + 'px ' + font;
             drawLabels(context, getLabels(obsCat, labelsPositions.labels, categoricalNames), labelsPositions.positions, chartOptions, chartSize, camera);
         }
     }
 
     function onSaveImage(format) {
-        const {trace, chartSize} = props;
         saveImage(trace, chartSize, drawContext, format);
     }
 
@@ -419,8 +508,8 @@ function ScatterChartThree(props) {
     function resetCamera() {
         const scatterPlot = scatterPlotRef.current;
         scatterPlot.resetZoom();
-        if (scatterPlot.interactionMode === 'PAN' && props.trace.dimensions === 3) {
-            props.onCamera('change', scatterPlot.getCameraDef());
+        if (scatterPlot.interactionMode === 'PAN' && trace.dimensions === 3) {
+            onCamera('change', scatterPlot.getCameraDef());
         }
     }
 
@@ -428,20 +517,20 @@ function ScatterChartThree(props) {
     function onShowAxis() {
         const scatterPlot = scatterPlotRef.current;
         const axes = scatterPlot.scene.getObjectByName('axes');
-        props.chartOptions.showAxis = !props.chartOptions.showAxis;
+        chartOptions.showAxis = !chartOptions.showAxis;
         if (axes) {
-            axes.visible = props.chartOptions.showAxis;
+            axes.visible = chartOptions.showAxis;
         }
 
-        props.setChartOptions(props.chartOptions);
+        setChartOptions(chartOptions);
     }
 
     function onShowFog() {
         const scatterPlot = scatterPlotRef.current;
-        props.chartOptions.showFog = !props.chartOptions.showFog;
+        chartOptions.showFog = !chartOptions.showFog;
         const spriteVisualizer = getVisualizer(scatterPlot, POINT_VISUALIZER_ID);
-        spriteVisualizer.styles.fog.enabled = props.chartOptions.showFog;
-        props.setChartOptions(props.chartOptions);
+        spriteVisualizer.styles.fog.enabled = chartOptions.showFog;
+        setChartOptions(chartOptions);
     }
 
 
@@ -456,94 +545,34 @@ function ScatterChartThree(props) {
             scatterPlot.rectangleSelector.setSelectionMode('LASSO');
             scatterPlot.setInteractionMode('SELECT');
         }
-        props.chartOptions.dragmode = mode;
-        props.setChartOptions(props.chartOptions);
+        chartOptions.dragmode = mode;
+        setChartOptions(chartOptions);
     }
 
     function onToggleAnimation() {
         const scatterPlot = scatterPlotRef.current;
         if (scatterPlot.orbitIsAnimating()) {
             scatterPlot.stopOrbitAnimation();
-            props.chartOptions.animating = false;
+            chartOptions.animating = false;
         } else {
             scatterPlot.startOrbitAnimation();
-            props.chartOptions.animating = true;
+            chartOptions.animating = true;
         }
-        props.setChartOptions(props.chartOptions);
+        setChartOptions(chartOptions);
 
     }
 
-    function cameraCallback(eventName) {
-        props.onCamera(eventName, scatterPlotRef.current.getCameraDef());
-    }
-
-    function init() {
-        if (scatterPlotRef.current == null) {
-            scatterPlotRef.current = createScatterPlot(containerElementRef.current, window.ApplePaySession, true);
-            if (props.chartOptions.dragmode === 'pan') {
-                scatterPlotRef.current.setInteractionMode('PAN');
-            } else if (props.chartOptions.dragmode === 'select') {
-                scatterPlotRef.current.setInteractionMode('SELECT');
-                scatterPlotRef.current.rectangleSelector.setSelectionMode('BOX');
-            } else if (props.chartOptions.dragmode === 'lasso') {
-                scatterPlotRef.current.setInteractionMode('SELECT');
-                scatterPlotRef.current.rectangleSelector.setSelectionMode('LASSO');
-            }
-            const axes = scatterPlotRef.current.scene.getObjectByName('axes');
-            if (axes) {
-                axes.visible = props.chartOptions.showAxis;
-                axes.setColors(props.chartOptions.darkMode ? new Color("rgb(255, 255, 255)") : new Color("rgb(0, 0, 0)"));
-            }
-            getVisualizer(scatterPlotRef.current, POINT_VISUALIZER_ID).styles.fog.enabled = props.chartOptions.showFog;
-            const canvas = containerElementRef.current.querySelector('canvas');
-            canvas.style.outline = '0px';
-            const webglcontextlost = (e) => {
-                console.log('lost webgl context');
-                e.preventDefault();
-                scatterPlotRef.current = null;
-            };
-            const webglcontextrestored = (e) => {
-                console.log('restored webgl context');
-                e.preventDefault();
-                setForceUpdate(!forceUpdate);
-            };
-            canvas.addEventListener('webglcontextlost', webglcontextlost);
-            canvas.addEventListener('webglcontextrestored', webglcontextrestored);
-            return true;
-        }
-        return false;
-
-    }
-
-
-    function draw() {
-        const {
-            obsCat,
-            cachedData,
-            trace,
-            markerOpacity,
-            unselectedMarkerOpacity,
-            selection,
-            pointSize,
-            chartOptions,
-            categoricalNames,
-            unselectedPointSize
-        } = props;
-        updateScatterChart(scatterPlotRef.current, trace, selection, markerOpacity, unselectedMarkerOpacity, pointSize, unselectedPointSize,
-            categoricalNames, chartOptions, obsCat, cachedData);
-    }
-
 
     return <>
-        <div className={props.classes.root}>
+        <div className={classes.root}>
             <ChartToolbar
-                dragmode={props.chartOptions.dragmode}
-                // editSelection={props.chartOptions.editSelection}
-                onGallery={props.onGallery}
-                animating={props.chartOptions.animating}
-                showFog={props.chartOptions.showFog}
+                dragmode={chartOptions.dragmode}
+                // editSelection={chartOptions.editSelection}
+                onGallery={onGallery}
+                animating={chartOptions.animating}
+                showFog={chartOptions.showFog}
                 onShowFog={onShowFog}
-                is3d={props.trace && props.trace.z != null}
+                is3d={trace && trace.z != null}
                 toggleAnimation={onToggleAnimation}
                 onSaveImage={onSaveImage}
                 onDragMode={onDragMode}
@@ -551,14 +580,15 @@ function ScatterChartThree(props) {
                 // onEditSelection={onEditSelection}
                 onShowAxis={onShowAxis}
                 onHome={resetCamera}
-                showAxis={props.chartOptions.showAxis}/>
+                showAxis={chartOptions.showAxis}/>
         </div>
         <div data-testid="scatter-chart-three" style={{
             display: 'inline-block',
-            width: props.chartSize.width,
-            height: props.chartSize.height
+            width: chartSize.width,
+            height: chartSize.height
         }}
              ref={containerElementRef}>
+            <CirroTooltip html={tip.html} clientX={tip.clientX} clientY={tip.clientY} transition={SCATTER_TRANSITION}/>
         </div>
     </>;
 
