@@ -8,29 +8,28 @@ import {Vector} from './Vector';
 export function getVarNameType(key) {
     let index = key.indexOf('/');
     if (index === -1) {
-        return {name: key, type: FEATURE_TYPE.X};
+        return {name: key, type: FEATURE_TYPE.X}; // default
     } else {
-        let key_type = key.substring(0, index);
-        let name = key.substring(index + 1);
-        return {name: name, type: key_type};
+        const type = key.substring(0, index);
+        const name = key.substring(index + 1);
+        return {name: name, type: type};
     }
 }
 
-function getStats(dimensions, obsMeasures, varMeasures) {
+function getStats(dimensionVectors, measureVectors) {
     const results = {};
-    dimensions.forEach(v => {
+    dimensionVectors.forEach(v => {
         results[v.getName()] = valueCounts(v);
     });
-    obsMeasures.forEach(v => {
-        results[v.getName()] = stats(v);
-    });
-    varMeasures.forEach(v => {
-        results[v.getName()] = stats(v);
+    measureVectors.forEach(v => {
+        results[v.getName()] = continuousVectorStats(v);
     });
+
     return results;
 }
 
-function getVectors(cachedData, names, indices = null) {
+
+export function getVectors(cachedData, names, indices = null) {
     let vectors = [];
     names.forEach(name => {
         let array = cachedData[name];
@@ -133,7 +132,13 @@ export function computeDerivedStats(result, q, cachedData) {
         const dimensions = q.stats.dimensions || [];
         const measures = q.stats.measures || [];
         const typeToMeasures = getTypeToMeasures(measures);
-        result.summary = getStats(getVectors(cachedData, dimensions), getVectors(cachedData, typeToMeasures.obs), getVectors(cachedData, typeToMeasures.X));
+
+        let measureVectors = [];
+        Object.values(typeToMeasures).forEach(fields => {
+            measureVectors = measureVectors.concat(getVectors(cachedData, fields));
+        });
+
+        result.summary = getStats(getVectors(cachedData, dimensions), measureVectors);
     }
 
     if (q.groupedStats) {
@@ -161,8 +166,9 @@ export function computeDerivedStats(result, q, cachedData) {
         result.selection = {};
         result.selection.indices = getPassingFilterIndices(cachedData, q.selection.filter);
         const selectedIndices = Array.from(result.selection.indices);
+        const dimensionVectors = getVectors(cachedData, dimensions, selectedIndices);
         if (dimensions.length > 0) {
-            const groupDimensionInfo = groupDimensions(getVectors(cachedData, dimensions, selectedIndices));
+            const groupDimensionInfo = groupDimensions(dimensionVectors);
             const distribution = {};
             for (const key in typeToMeasures) {
                 const measures = typeToMeasures[key];
@@ -172,8 +178,11 @@ export function computeDerivedStats(result, q, cachedData) {
             }
             result.selection.distribution = distribution;
         }
-        result.selection.summary = getStats(getVectors(cachedData, dimensions, selectedIndices), getVectors(cachedData, typeToMeasures.obs, result.selection.indices),
-            getVectors(cachedData, typeToMeasures.X, selectedIndices));
+        let measureVectors = [];
+        Object.values(typeToMeasures).forEach(fields => {
+            measureVectors = measureVectors.concat(getVectors(cachedData, fields, selectedIndices));
+        });
+        result.selection.summary = getStats(dimensionVectors, measureVectors);
     }
 }
 
@@ -224,7 +233,7 @@ export function groupedStats(groupDimensionInfo, vectors) {
         const value = groupDimensionInfo.keyToIndices[category];
         vectors.forEach((v) => {
             const categoryVector = new SlicedVector(v, value.indices);
-            const categoryStats = stats(categoryVector);
+            const categoryStats = continuousVectorStats(categoryVector);
             const entry = {
                 dimension: groupDimensionInfo.dimensionName,
                 dimensions: groupDimensionInfo.dimensionNames,
@@ -261,16 +270,20 @@ export function toFloatArray(v) {
 export function variance(v, mean) {
     const size = v.size();
     if (size <= 1) {
-        return NaN;
+        return Number.NaN;
     }
     let ss = 0;
+    let count = 0;
     for (let j = 0; j < size; j++) {
         let x = v.get(j);
-        let diff = x - mean;
-        diff = diff * diff;
-        ss += diff;
+        if (!Number.isNaN(x)) {
+            let diff = x - mean;
+            diff = diff * diff;
+            ss += diff;
+            count++;
+        }
     }
-    let n = size - 1;
+    let n = count - 1;
     if (n < 1) {
         n = 1;
     }
@@ -278,7 +291,7 @@ export function variance(v, mean) {
 }
 
 
-export function stats(v) {
+export function continuousVectorStats(v) {
 
     if (v.size() === 0) {
         return {
@@ -295,18 +308,22 @@ export function stats(v) {
     let sum = 0;
     let numExpressed = 0;
     let logSum = 0;
+    let count = 0;
     for (let i = 0, size = v.size(); i < size; i++) {
         const value = v.get(i);
-        min = value < min ? value : min;
-        max = value > max ? value : max;
-        sum += value;
-        logSum += Math.expm1(value);
-        if (value !== 0) {
-            numExpressed++;
+        if (!Number.isNaN(value)) {
+            count++;
+            min = value < min ? value : min;
+            max = value > max ? value : max;
+            sum += value;
+            logSum += Math.expm1(value);
+            if (value !== 0) {
+                numExpressed++;
+            }
         }
     }
 
-    const mean = sum / v.size();
+    const mean = sum / count;
     return {
         min: min, max: max, sum: sum, mean: mean, numExpressed: numExpressed, n: v.size(), logSum: logSum
     };
